/* StringLight Timer using ESP8266 (NodeMCU 1.0)
 */
 
#include <ESP8266WiFi.h>
#include <ESP8266HTTPClient.h>
#include <WiFiClient.h>
#include <NTPClient.h>
#include <WiFiUdp.h>
#include <time.h>
#include <Timezone.h>
#include <string.h>
#include <ArduinoJson.h>

const int buttonPin = 4; // GPIO4 = D2
String LAT = "YOUR_LAT";
String LNG = "YOUR_LNG";
String server =    "http://api.sunrise-sunset.org";
String resource =  "/json?lat=" + LAT + "&lng=" + LNG + "&date=today";
String ssid = "YOUR_SSID";
String psswd = "YOUR_PSSWD";
int utcOffset = -7; // GMT to PDT (-8 for PST)
const long utcOffsetInSeconds = utcOffset*60*60;
String API_return;
String current_time;
    
void setup()
{
  // Connect to Wifi network
  Serial.begin(115200);
  pinMode(buttonPin, OUTPUT);
  
  wifiConnect();
  int currentTime = getCurrentTime();
  int sunset = getSunset();

  // Turn lights on 40 minutes before sunset and keep lights on for 2 hours
  if ((currentTime >= sunset -40) && (currentTime < sunset)){
    WiFi.mode(WIFI_OFF);
    
  // Flash RF button for 1s (Turn lights ON)
    Serial.println("START");
    digitalWrite(buttonPin, HIGH);
    delay(1*1000);
    digitalWrite(buttonPin, LOW);

    //Turn lights off after 2 hour delay
    /* For some reason, delay function stalls when replacing
     *  60 with 120, but works correctly when the 2 hour delay 
     *  is broken into two separate lines (1 hour delay each) */
    delay(60*60*1000UL);
    delay(60*60*1000UL);
    
  // Flash RF button for 1s (Turn lights OFF)
    Serial.println("STOP");
    digitalWrite(buttonPin, HIGH);
    delay(1*1000);
    digitalWrite(buttonPin, LOW);
  }
  
  // Put ESP8266 into Deep-sleep mode to save power
  ESP.deepSleep(10*60*1e6); //10 minutes measured in microseconds
}

void loop() {
  }

void wifiConnect(){
  Serial.println();
  WiFi.begin(ssid, psswd);
  Serial.print("Connecting");
  while (WiFi.status() != WL_CONNECTED){
    delay(500);
    Serial.print(".");
  }
  Serial.println("Connected!");
}

// Function for acquiring sunset time from API call
int getSunset(){
  StaticJsonDocument<768> doc; //Memory size estimated from https://arduinojson.org/v6/assistant/
  String serverPath = server + resource;
  API_return = httpGETRequest(serverPath);
 
  // Deserialize the JSON document
  DeserializationError error = deserializeJson(doc, API_return);

  // Test if parsing succeeds
  if (error) {
    Serial.print(F("deserializeJson() failed: "));
    Serial.println(error.f_str());
    //return;
  }
  JsonObject results = doc["results"];
  const char* results_sunset = results["sunset"]; // e.g. "2:58:31 AM"

  // Print values
  Serial.println("Sunset time: ");
  Serial.println(results_sunset);

  // Convert char results to integer (split hour/min from timestamp)
  char data[40];
  strcpy(data, results_sunset);
  char* sunset_hour = strtok(data," ,.:");
  char* sunset_min = strtok(NULL," ,.:");
  
  int sunset_int = (String(sunset_hour).toInt()+24+utcOffset)*60 + String(sunset_min).toInt();
  return sunset_int;
}

int getCurrentTime(){
  // Define NTP Client to get time
  WiFiUDP ntpUDP;
  NTPClient timeClient(ntpUDP, "north-america.pool.ntp.org", utcOffsetInSeconds);
  timeClient.begin();
  timeClient.update();
  unsigned long epoch = timeClient.getEpochTime();
  setTime(epoch);
  /*
  Serial.print("Current Time:  ");
  Serial.print(timeClient.getHours());
  Serial.print(":");
  Serial.print(timeClient.getMinutes());
  Serial.print(":");
  Serial.println(timeClient.getSeconds());
  */

  int hour_int = String(timeClient.getHours()).toInt();
  int min_int  = String(timeClient.getMinutes()).toInt();
  int time_int = 60*hour_int + min_int;
  return time_int;
}

// Http GET request function
String httpGETRequest(String serverName) {
  HTTPClient http; 
  http.begin(serverName);
  // Send HTTP POST request
  int httpResponseCode = http.GET();
  String payload = "{}"; 
  if (httpResponseCode>0) {
    payload = http.getString();
  }
  else {
    Serial.print("Error code: ");
    Serial.println(httpResponseCode);
  }
  // Free resources
  http.end();
  return payload;
}
